﻿/* Copyright © Abouzakaria
 * (c)GPL2 (c)GPL3
 *
 * This file is part of booksorg.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License version 2 or at your option version 3 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */
#include "mainwindow.h"

#include "eicon.h"
#include "ui_mainwindow.h"
#include "addfolderwizard.h"
#include "editbookdialog.h"
#include "addbookwizard.h"
#include "bookinfo.h"
#include <QDebug>
#include <QInputDialog>
#include <QDirIterator>
#include <QDesktopServices>
#include <QUrl>
#include <QFileDialog>
#include <QMessageBox>
#include <QProgressDialog>
#include <QFontMetrics>
#include <QErrorMessage>
#include <QImageReader>
#include <QTextCodec>
#include <QFileInfo>
#define thumbnails QDir::homePath()+"/."+QApplication::applicationName()+"/thumbnails"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{

    QTextCodec *codec = QTextCodec::codecForName("UTF-8");
    QIcon::setThemeName(EIcon::themeName());
    ui->setupUi(this);


ui->labelImageInfo->setPixmap(QPixmap(style()->standardPixmap(QStyle::SP_MessageBoxInformation)));
    setAcceptDrops(true);

    mModel=new MyModel;

    mSearchTools =new SearchTools(false);

    mActions=new Actions(this->locale());

    mRatingWidget =new RatingWidget;
//ui->tabWidget->setCornerWidget(ui->mainToolBar,Qt::TopLeftCorner);

    ui->hLayoutRattings->addWidget(mRatingWidget);
     connect(mRatingWidget,SIGNAL(ratingChanged(int,QString)),mModel,SLOT(saveRating(int,QString)));

    connect(mActions,SIGNAL(categoryChanged(int)),mModel,SLOT(setCategory(int)));
    connect(mActions,SIGNAL(viewModeChanged(int)),ui->stackedWidget,SLOT(setCurrentIndex(int)));
    connect(mActions,SIGNAL(emitDeletSelectedIndexes()),this,SLOT(deletSelectedIndexes()));
    connect(mActions,SIGNAL(emitEditBook()),this,SLOT(editBook()));
    connect(mActions,SIGNAL(emitRootPath()),this,SLOT(goRootPath()));

    connect(this,SIGNAL(curPageChanged(int)),mActions,SIGNAL(currentPageChanged(int)));
 //   connect(mActions,SIGNAL(pdfToggleContentVisible(bool)),this,SLOT(wPdfToggleContentVisible(bool)));
    connect(mActions,SIGNAL(pdfEmitAddBookmark()),this,SLOT(wPdfEmitAddBookmark()));
    connect(mActions,SIGNAL(pdfEmitFirstPage()),this,SLOT(wPdfEmitFirstPage()));
    connect(mActions,SIGNAL(pdfEmitLastPage()),this,SLOT(wPdfEmitLastPage()));
    connect(mActions,SIGNAL(pdfEmitNextPage()),this,SLOT(wPdfEmitNextPage()));
    connect(mActions,SIGNAL(pdfEmitPreviousPage()),this,SLOT(wPdfEmitPreviousPage()));
    connect(mActions,SIGNAL(pdfEmitSetInvertColors(bool)),this,SLOT(wPdfEmitSetInvertColors(bool)));
    connect(mActions,SIGNAL(pdfEmitSetRubberBand(bool)),this,SLOT(wPdfEmitSetRubberBand(bool)));
    connect(mActions,SIGNAL(pdfEmitZoomIn()),this,SLOT(wPdfEmitZoomIn()));
    connect(mActions,SIGNAL(pdfEmitZoomOut()),this,SLOT(wPdfEmitZoomOut()));
    connect(mActions,SIGNAL(pdfSetContinousMode(bool)),this,SLOT(wPdfSetContinousMode(bool)));
    connect(mActions,SIGNAL(pdfSetFiteWith(int)),this,SLOT(wPdfSetFiteWith(int)));
    connect(mActions,SIGNAL(pdfSetLayoutMode(int)),this,SLOT(wPdfSetLayoutMode(int)));
    connect(mActions,SIGNAL(pdfSetPage(int)),this,SLOT(wPdfSetPage(int)));
    connect(mActions,SIGNAL(pdfRefresh()),this,SLOT(wPdfRefresh()));

    setupTools();

    ui->dockWidgetBookmark->setVisible(false);

    QFontMetrics fm(ui->listView->font());
    ui->listView->setGridSize(QSize(135,135+(fm.height()*2)));

    ui->listView->setModel(mModel);
    ui->treeView->setModel(mModel);

    ui->treeView->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(ui->treeView,SIGNAL(customContextMenuRequested(QPoint)),this,SLOT(customContextMenu(QPoint)));
    ui->listView->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(ui->listView,SIGNAL(customContextMenuRequested(QPoint)),this,SLOT(customContextMenu(QPoint)));

    connect( ui->listView,SIGNAL(activated(QModelIndex)),this,SLOT(slotItemActivated(QModelIndex)));
    connect( ui->treeView,SIGNAL(activated(QModelIndex)),this,SLOT(slotItemActivated(QModelIndex)));
    connect( ui->listView->selectionModel(),SIGNAL(currentRowChanged(QModelIndex,QModelIndex))
             ,this,SLOT(slotcurrentRowChanged(QModelIndex,QModelIndex)));
    connect( ui->treeView->selectionModel(),SIGNAL(currentRowChanged(QModelIndex,QModelIndex))
             ,this,SLOT(slotcurrentRowChanged(QModelIndex,QModelIndex)));

    connect(mModel,SIGNAL(pathChanged(QString)),ui->statusBar,SLOT(showMessage(QString)));
    connect(mModel,SIGNAL(pathChanged(QString)),this,SLOT(pathChanged(QString)));

    connect(mSearchTools,SIGNAL(searchPressed(QString)),this,SLOT(goSearch(QString)));
    //-----------------------------------------------------
    mWidgetBookmark=new WidgetBookmark;
    ui->vLayoutBookmark->addWidget(mWidgetBookmark);
    connect(mWidgetBookmark,SIGNAL(getBookmark()),this,SLOT(slotAddBookmark()));
    connect(mWidgetBookmark,SIGNAL(openDocument(QString,int,qreal,qreal))
            ,this,SLOT(bookmarkOpenDocument(QString,int,qreal,qreal)));
    //---------------------------------------------

    QStringList args = QApplication::arguments();

    if(args.count()>1)
    {
        QUrl url = args.at(1);
        //  QMessageBox::information(this,"",url.toLocalFile());

        qDebug()<<"url:"<<url;
        QByteArray encodedString =QByteArray::fromPercentEncoding(url.toString().toUtf8());
        QString string = codec->toUnicode(encodedString);

        QFileInfo fi(string);

        qDebug()<<"fi"<<fi.absoluteFilePath();

        if (fi.exists() && fi.suffix().toLower()=="pdf" )
        {
            qDebug() << fi.filePath()  ;
            slotOpenFileName(fi.filePath());
        }

    }

    //---------------------------------------------
    QSettings settings;
    settings.beginGroup("Main");
    restoreGeometry(settings.value("Geometry").toByteArray());
    settings.endGroup();
      qDebug()<<"loadGeometry";
    //---------------------------------------------
setCurentTab(0);
//if(isRightToLeft())
//      ui->dockWidgetBookmark->setAllowedAreas(Qt::RightDockWidgetArea);
if(QApplication::layoutDirection()==Qt::RightToLeft)
    addDockWidget(Qt::RightDockWidgetArea,ui->dockWidgetBookmark);

}

MainWindow::~MainWindow()
{
    qDebug()<<"saveGeometry";
    QSettings settings;
    settings.beginGroup("Main");
    settings.setValue("Geometry", saveGeometry());
    settings.endGroup();

    delete ui;

}

void MainWindow::pathChanged(QString)
{

    if(focusWidget()==ui->treeView)
    {
        ui->treeView->resizeColumnToContents(0);
        ui->treeView->resizeColumnToContents(4);
    }

}

void MainWindow::setupTools()
{

    QSettings setting;



    ui->hLayoutBooks->addWidget(mSearchTools);
    mSearchTools->setVisible(false);
    /*!
            انشاء الاوامر اللازمة
            ثم اظافة الاوامر الى القائمة واللوحة
     */
    mActBooksTab=new QAction(tr("Books"),this);
    mActBooksTab->setStatusTip(tr("show books tab"));
    mActBooksTab->setIcon(EIcon::icon(BOOKS_TAB));
    mActBooksTab->setShortcut(QKeySequence(QKeySequence::AddTab));

    mActScanFolder=new QAction(tr("Scan folder"),this);
    mActScanFolder->setStatusTip(tr("scan folder for existing books"));
    mActScanFolder->setIcon(EIcon::icon(WIZARD));
    mActScanFolder->setShortcut(QKeySequence("Ctrl+W"));

    mActOpen=new QAction(tr("Open"),this);
    mActOpen->setStatusTip(tr("Open... "));
    mActOpen->setIcon(EIcon::icon(OPEN));

    mActOpenWith=new QAction(tr("Open With..."),this);
    mActOpenWith->setStatusTip(tr("Open book with default programme"));
    mActOpenWith->setIcon(EIcon::icon(OPEN));

    mActMoveToAuthor=new QAction(tr("Change Author"),this);
    mActMoveToAuthor->setStatusTip(tr("Change Author for selected books"));
    mActMoveToAuthor->setIcon(EIcon::icon(AUTORS));

    mActMoveToGroup=new QAction(tr("Change Group"),this);
    mActMoveToGroup->setStatusTip(tr("Change Group for selected books"));
    mActMoveToGroup->setIcon(EIcon::icon(GROOPS));

    mActAddBook=new QAction(tr("Add book"),this);
    mActAddBook->setStatusTip(tr("Add new book"));
    mActAddBook->setIcon(EIcon::icon(ADD));
    mActAddBook->setShortcut(QKeySequence::Open);
    mActAddBook->setToolTip(tr("Add new book")+"\n"+mActAddBook->shortcut().toString());

    mActAbout=new QAction(tr("About"),this);
    mActAbout->setStatusTip(tr("About... "));

    // find------------------
    mActFind= new QAction(EIcon::icon(FIND),tr("&Find"),this);
    connect(mActFind,SIGNAL(triggered()),this,SLOT(showHideSearch()));
    mActFind->setShortcut(QKeySequence::Find);
    mActFind->setToolTip(mActFind->toolTip()+"\n"+mActFind->shortcut().toString());
    mActFind->setStatusTip(mActFind->toolTip().replace("\n","  "));


    //View Mode Actions
    int vMode=setting.value("ViewMode",0).toInt();

    ui->stackedWidget->setCurrentIndex(vMode);

    // Category Actions
    int cat=setting.value("Category",0).toInt();

    mModel->setCategory(cat);
    //recent
    menuRecent=new QMenu(tr("Recent files"),this);
    mActRecents=new QAction(EIcon::icon(RECENT),tr("Recent files"),this);
    for (int i = 0; i < MaxRecentFiles; ++i) {
        recentFileActs[i] = new QAction(this);
        recentFileActs[i]->setVisible(false);
        connect(recentFileActs[i], SIGNAL(triggered()),
                this, SLOT(openRecentFile()));
        menuRecent->addAction(recentFileActs[i]);
    }

    mActRecents->setMenu(menuRecent);
    QAction *actBookmark=ui->dockWidgetBookmark->toggleViewAction();
    //Cnnections
    connect(mActRecents,SIGNAL(triggered()),recentFileActs[0],SLOT(trigger()));
    connect(mActBooksTab,SIGNAL(triggered()),this,SLOT(showTabBooks()));
    connect(mActAddBook,SIGNAL(triggered()),this,SLOT(addBook()));
    connect(mActOpen,SIGNAL(triggered()),this,SLOT(slotOpen()));
    connect(mActOpenWith,SIGNAL(triggered()),this,SLOT(slotOpenWith()));
    connect(mActScanFolder,SIGNAL(triggered()),this,SLOT(scanFolder()));
    connect(ui->tabWidget, SIGNAL(currentChanged(int)),this, SLOT(setCurentTab(int)));
    connect(mActMoveToAuthor, SIGNAL(triggered()),this, SLOT(changeAuthor()));
    connect(mActMoveToGroup, SIGNAL(triggered()),this, SLOT(changeGroup()));
    connect(mActAbout, SIGNAL(triggered()),this, SLOT(slotAbout()));

    //menu file
    ui->menuFile->addAction(mActAddBook);
    ui->menuFile->addAction(mActScanFolder);
    ui->menuFile->addAction(mActBooksTab);
    ui->menuFile->addSeparator();
    ui->menuFile->addMenu(menuRecent);
    ui->menuFile->addSeparator();
    ui->menuFile->addAction(EIcon::icon(QUITE),tr("&Quit"),this,SLOT(close()));
    //menu edit
    ui->menuEdit->addAction(mActFind);
    ui->menuEdit->addActions( mActions->listEditBooks());
    ui->menuEdit->addSeparator();
    ui->menuEdit->addActions( mActions->listCatBooks());
    ui->menuEdit->addSeparator();
    ui->menuEdit->addActions( mActions->listEditPdf());
    ui->menuEdit->addSeparator();
    ui->menuEdit->addAction(actBookmark);
    //menu view
    ui->menuView->addActions(mActions->listViewBooks());
    ui->menuView->addActions(mActions->listViewPdf());
    //menu help
    ui->menu_Help->addAction(mActAbout);
    QMenu *menuLang=ui->menu_Help->addMenu(tr("Switch Language..."));
    QActionGroup *actAlignLang=new QActionGroup(this);

    QSettings settings;
    settings.beginGroup("Main");
    QString locale= settings.value("Lang", "Default").toString();
    settings.endGroup();

    QAction *a=menuLang->addAction(tr("Default"));
    a->setCheckable(true);
    a->setData("Default");
    a->setChecked(locale=="Default");
    actAlignLang->addAction(a);

    a=menuLang->addAction(tr("Arabic"));
    a->setCheckable(true);
    a->setData("ar");
    a->setChecked(locale=="ar");
    actAlignLang->addAction(a);

    a=menuLang->addAction(tr("English"));
    a->setCheckable(true);
    a->setChecked(locale=="en");
    a->setData("en");
    actAlignLang->addAction(a);

    a=menuLang->addAction(tr("Francais"));
    a->setCheckable(true);
    a->setChecked(locale=="fr");
    a->setData("fr");
    actAlignLang->addAction(a);

    connect(actAlignLang,SIGNAL(triggered(QAction*))
            ,this,SLOT(swichLanguage(QAction*)));

    //tool bar
    ui->mainToolBar->addAction(mActRecents);
    ui->mainToolBar->addAction(mActBooksTab);

    actBookmark->setIcon(EIcon::icon(BOOKMARK));
    ui->mainToolBar->addAction(actBookmark);

    ui->mainToolBar->addSeparator();
    ui->mainToolBar->addActions( mActions->listToolBarBooks());
    ui->mainToolBar->addActions( mActions->listToolBarPdf());

    updateRecentFileActions();
}

//______________________________________________________________________
void MainWindow::setCurentTab(int /*arg*/)
{
    /*!
            جعل بعض الاوامر مرئية والاخرى غير مرئية
            في حالة تغيير التبويب بين عرض الكتب
            وعرض الكتاب
     */
 //BooKs tools
 bool visible=ui->tabWidget->currentWidget()!=ui->tabBooks;
// qDebug()<<visible<<"tabBooks";
    foreach (QAction *act, mActions->listToolBarBooks()) {
        act->setVisible(!visible);
   }

    //Pdf view tools
    foreach (QAction *act, mActions->listViewPdf()) {
        act->setVisible(visible);
    }

    foreach (QAction *act, mActions->listEditPdf()) {
        act->setVisible(visible);
    }
    foreach (QAction *act, mActions->listToolBarPdf()) {
        act->setVisible(visible);
    }

    if(visible){
        WidgetPdf   *wPdf=widgetPdf();
        if(wPdf){

            mActions->setSpinPages(wPdf->numPages(),wPdf->curPage());
          //  mActions->setPdfContentVisibleAct(wPdf->isContentVisible());
            mActions->setRubberBandAct(wPdf->isRubberBand());
            mActions->setFitSizeAct(wPdf->docFiteWith());
            mActions->setLayoutModeAct(wPdf->doctLayoutMode());
            mActions->setContinousModeAct(wPdf->isContinousMode());
            mActions->setInvertcolorsAct(wPdf->isInvertColors());

        }
    }
        ui->tabWidget->setTabsClosable(ui->tabWidget->count()-1);

}

//______________________________________________________________________Add
void MainWindow::scanFolder()
{

    /*!
     * مسح مجلد بحثا عن ملفات كتب
     */

    AddFolderWizard *wiz=new AddFolderWizard(mModel->groupList(),mModel->authorList(),this);

    if( wiz->exec()==QDialog::Accepted)
    {

        QList<BookInfo *> list=wiz->getListBook();

        QProgressDialog progress(tr("Create books information..."), tr("Cancel"), 0, list.count(), this);
        progress.setWindowModality(Qt::WindowModal);
        progress.setCancelButton(0);
        progress.show();
        qApp->processEvents();
        int i=0;
        foreach (BookInfo *b, list) {
            i++;
            progress.setValue(i);
            mModel->addBook(b->title(),b->author(),b->group(),b->icon()
                            ,b->comment(),b->properties(),b->path());
            qApp->processEvents();

        }

    }

    delete wiz;
    mModel->refreshPath();

}

//______________________________________________________________________Add
void MainWindow::addBook()
{

    AddBookWizard *wiz=new AddBookWizard(mModel->groupList(),mModel->authorList(),this);

    if( wiz->exec()==QDialog::Accepted)
    {
        QMap<QString,QString>map=wiz->getNewBook();
        // qDebug()<<map;
        mModel->addBook(map["Name"],map["Author"],map["Group"],map["Icon"]
                ,map["Comment"],map["Properties"],map["Exec"]);


        slotOpenFileName(map["Exec"]);
        mModel->refreshPath();
    }

    delete wiz;

}

void MainWindow::savRating(int r,QString file)
{


    qDebug()<<"save"<<r<<file;
    //mModel->saveRating(file,r);
}

//______________________________________________________________________Edit
void MainWindow::editBook()
{
    /*!
     * تحرير الكتاب المحدد
     */

    QModelIndex index=currentIndex();

    if(!index.isValid())return;

    EditBookDialog *dlg=new EditBookDialog(mModel->isCategory());
    QString title=mModel->title(index);
    dlg->setTitle(title);
    dlg->setIcon(mModel->item(index.row(),0)->icon());
    if(!mModel->isCategory()){
        dlg->setComment(mModel->comment(index));
        dlg->setAuthor(mModel->authorList(),mModel->author(index));
        dlg->setRating(mModel->favorite(index).toInt());
        dlg->setPath(mModel->path(index));
        dlg->setGroup(mModel->groupList(),mModel->group(index));
    }

    if(dlg->exec()==QDialog::Accepted){
        if(mModel->isCategory()){
            mModel->updateCategiry(title,dlg->title());
        }else{
            mModel->updateBook(index,dlg->title(),dlg->author()
                               ,dlg->group(),dlg->comment()
                               ,dlg->path(),dlg->rating());
        }
    }

    delete dlg;

}

//______________________________________________________________________Delete
void MainWindow::deletSelectedIndexes()
{


    QModelIndexList selectedIndexes;
    if (focusWidget() == ui->listView){
        selectedIndexes=ui->listView->selectionModel()->selectedIndexes();
    }else if (focusWidget() == ui->treeView){
        selectedIndexes=ui->treeView->selectionModel()->selectedIndexes();
    }
    //  qDebug()<<selectedIndexes.count();

    mModel->deleteSelectedIndexes(selectedIndexes);

}

//______________________________________________________________________
void MainWindow::changeAuthor()
{

    moveItems(true);

}

void MainWindow::changeGroup()
{

    moveItems(false);
}

void MainWindow::moveItems(bool authors)
{

    QModelIndexList selectedIndexes;
    if (focusWidget() == ui->listView){
        selectedIndexes=ui->listView->selectionModel()->selectedIndexes();
    }else if (focusWidget() == ui->treeView){
        selectedIndexes=ui->treeView->selectionModel()->selectedIndexes();
    }
    if(selectedIndexes.count()<1)return;

    QStringList items;
    QString name;
    if(authors){
        items=mModel->authorList();
        name=tr("Author");
    }else{
        items=mModel->groupList();
        name=tr("Group");
    }

    bool ok;
    QString item = QInputDialog::getItem(this, tr(""),
                                         tr("Select %1:").arg(name), items, -1, true, &ok);
    if (!ok || item.isEmpty())return;

    QList<int>list;
    QProgressDialog progress(tr("Change informations..."), tr("Cancel"), 0
                             , selectedIndexes.count()/5, this);
    progress.setWindowModality(Qt::WindowModal);
    progress.setCancelButton(0);
    progress.show();
    qApp->processEvents();
    int i=0;
    foreach (QModelIndex index, selectedIndexes)
    {

        if(!list.contains( index.row())){
            list.append(index.row());
            i++;
            progress.setValue(i);

            mModel->changeParent(index,item,authors);

            qApp->processEvents();
        }
    }

    mModel->refreshPath();

}

//______________________________________________________________________
void MainWindow::goRootPath()
{

    mModel->setCategory(mModel->category());
    ui->treeView->resizeColumnToContents(0);

}

//__________________________________________________________________open
void MainWindow::slotItemActivated(const QModelIndex &index)
{

    if(mModel->isRoot(index)){

        QString path=mModel->title(index);
        mModel->setRootPath(path);

    }else{

        QString fileName=mModel->path(index);
        slotOpenFileName(fileName);

    }

}

//__________________________________________________________________open
void MainWindow::slotOpen()
{

    QModelIndex index=currentIndex();
    if(index.isValid())
        slotItemActivated(index);

}

//__________________________________________________________________open
void MainWindow::slotOpenWith()
{

    QModelIndex index=currentIndex();
    if(!index.isValid())return;

    QString fileName=mModel->path(index);
    QDesktopServices::openUrl(QUrl::fromLocalFile(fileName));

}

//__________________________________________________________________open
void MainWindow::openRecentFile()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action)
        slotOpenFileName(action->data().toString());

}

//__________________________________________________________________open
void MainWindow::slotOpenFileName(const QString &fileName)
{

    int page=mModel->page(fileName);

     for (int i = 0; i < ui->tabWidget->count(); ++i)
    {

        WidgetPdf   *w= qobject_cast<WidgetPdf *>( ui->tabWidget->widget(i));
        if(w)
        {

            if(w->filePath()==fileName)
            {
                qDebug()<<"openrd" << fileName ;
                ui->tabWidget->setCurrentWidget(w);

                return;
            }

        }

    }

    WidgetPdf    *widgetPdf=new WidgetPdf;


    if(widgetPdf->openDoc(fileName,page))
    {

        ui->tabWidget->addTab(widgetPdf,QIcon(":/icons/application-pdf.png"),  bookTitle(fileName,widgetPdf));

        ui->tabWidget->setCurrentWidget(widgetPdf);
        connect(widgetPdf,SIGNAL(pageChanged(QString,int)),mModel,SLOT(savePage(QString,int)));
        connect(widgetPdf,SIGNAL(curPageChanged(int)),this,SIGNAL(curPageChanged(int)));

        connect(widgetPdf,SIGNAL(addNewBookmark(QString,int,qreal,qreal,QString)),
                mWidgetBookmark,SLOT(addBookmark(QString,int,qreal,qreal,QString)));

    }
    else
    {

        delete widgetPdf;

    }

}

//________________________________________________________________________
QString MainWindow::bookTitle(const QString &fileName,WidgetPdf *w)
{
    QString title=mModel->title(fileName);

    if(title.isEmpty()&&w){


        QMap<QString,QString>map=w->docInfo();

        mModel->addBook(map["Name"],map["Author"],map["Group"],map["Icon"]
                ,map["Comment"],map["Properties"],map["Exec"]);
        mModel->refreshPath();

        title=map["Name"];
        // qDebug()<<"doc is emty"<<map["Name"];

    }

    if(title.isEmpty()){
        title=QFileInfo(fileName).baseName();
    }
    if(title.length()>40){
        title= title.left(37);
        title=title.leftJustified(40,'.');
    }

       //save recent
    QSettings settings;
    settings.sync();
    QStringList files = settings.value("RecentFileList").toStringList();
    files.removeAll(fileName);
    files.prepend(fileName);
    while (files.size() > MaxRecentFiles)
        files.removeLast();

    settings.setValue("RecentFileList", files);
    updateRecentFileActions();

    return title;

}

//____________________________________________________________________________________________________
void MainWindow::slotcurrentRowChanged(const QModelIndex & current, const QModelIndex & /*previous*/ )
{

    if(mModel->isRoot(current))
    {

//        ui->labelStars->setPixmap(QPixmap());
        ui->labelImageInfo->setPixmap(mModel->item(current.row(),0)->icon().pixmap(256));
        ui->labelCommentInfo->setText(QString());
        ui->labelAuthorInfo->setText(QString());
        ui->textEditPrprtiesInfo->setText(QString());
mRatingWidget->clearRating();
    }
    else
    {

//        ui->labelStars->setPixmap(QString(":/icons/%1.png")
//                                  .arg(mModel->favorite(current)));
        int rating=mModel->favorite(current).toInt();
        qDebug()<<rating;
        mRatingWidget->setRating(rating,mModel->path(current));
        QPixmap pix(thumbnails+"/"+mModel->pixmap(current));
        ui->labelImageInfo->setPixmap(pix);

        ui->labelCommentInfo->setText(mModel->comment(current));
        ui->labelAuthorInfo->setText(mModel->author(current));
        QString prop=mModel->properties(current);
        prop.replace("CreationDate",tr("Created"));
        prop.replace("Title",tr("Title"));
        prop.replace("Author",tr("Author"));
        prop.replace("Creator",tr("Creator"));
        prop.replace("Producer",tr("Producer"));
        prop.replace("ModDate",tr("Modified"));
        prop.replace("Format",tr("Format"));
        prop.replace("Pages",tr("Pages"));
        prop+="\n"/*+tr("Path:")*/+mModel->path(current);
        ui->textEditPrprtiesInfo->setText(prop);

    }


    QString title=mModel->title(current);
    ui->labelTitleInfo->setText(title);

    ui->statusBar->showMessage(mModel->pathToString()+"/"+title);
}

//______________________________________________________________________
void MainWindow::customContextMenu(QPoint)
{
    QModelIndex index=currentIndex();
    int count=0;
    if (focusWidget() == ui->listView){
        count=ui->listView->selectionModel()->selectedIndexes().count();
    }else if (focusWidget() == ui->treeView){
        count=ui->treeView->selectionModel()->selectedIndexes().count();
    }

    QMenu menu;
    if(count>0&&index.isValid())
    {

        menu.addAction(mActOpen);
        if(!mModel->isRoot(index))
            menu.addAction(mActOpenWith);

        menu.addSeparator();
//        menu.addAction(mActEditBook);
//        menu.addAction(mActDelete);
    menu.addActions(    mActions->listEditBooks());
        if(!mModel->isRoot(index)){
            menu.addAction(mActMoveToAuthor);
            menu.addAction(mActMoveToGroup);
        }



    }
    menu.addSeparator();
    menu.addAction(mActAddBook);

    menu.exec(QCursor::pos());

}

//______________________________________________________________________
void MainWindow::toggleView(QAction* action)
{
    int mode=action->data().toInt();
    ui->stackedWidget->setCurrentIndex(mode);
    QSettings setting;
    setting.setValue("ViewMode",mode);
}

//______________________________________________________________________
void MainWindow::toggleRootPath(QAction* action)
{
    int cat=action->data().toInt();
    mModel->setCategory(cat);
    QSettings setting;
    setting.setValue("Category",cat);
}

//______________________________________________________________________
QModelIndex MainWindow::currentIndex()
{
    QModelIndex index;
    if (focusWidget() == ui->listView){
        index=ui->listView->currentIndex();
    }else if (focusWidget() == ui->treeView){
        index=ui->treeView->currentIndex();
    }
    return index;
}


//________________________________________________________Search
void MainWindow::goSearch(const QString &arg)
{
    mModel->setSearchPath(arg,mSearchTools->isMachCase());
}

//_________________________________________________________recent
void MainWindow::updateRecentFileActions()
{

    QSettings settings;;

    settings.sync();
    QStringList files = settings.value("RecentFileList").toStringList();

    int numRecentFiles = qMin(files.size(), (int)MaxRecentFiles);

    for (int i = 0; i < numRecentFiles; ++i) {
        QString name=mModel->title(files[i]);
        name=name!=QString() ? name:QFileInfo(files[i]).baseName();
        QString text = tr("&%1 %2").arg(i + 1).arg(name);
        recentFileActs[i]->setText(text);
        recentFileActs[i]->setData(files[i]);
        recentFileActs[i]->setVisible(true);

    }

    for (int j = numRecentFiles; j < MaxRecentFiles; ++j)
        recentFileActs[j]->setVisible(false);

}

//_________________________________________________________dragdrop
void MainWindow::dragEnterEvent(QDragEnterEvent *e)
{
    // Getting URL
    if (e->mimeData()->hasUrls())
    {
        e->acceptProposedAction();

    }


}

//_________________________________________________________dragdrop
void MainWindow::dropEvent(QDropEvent *e)
{
    /*!
           * فتح الكتاب في حالة البيانات كانت عبارة عن
           * ملفات ادلة
           * ثم فتح الملف الاول فقط عند تحقق شرط كونها مافات كتب بلاحقة
           * pdf
           */

    const QMimeData *mime = e->mimeData();

    QList<QUrl> duplicates;

    foreach (QUrl url, mime->urls())
    {

        if (duplicates.contains(url))
            continue;
        else
            duplicates << url;

        qDebug()<<url;
        QString fileName(url.toLocalFile());
        QFileInfo fi(fileName);

        if (fi.exists() && fi.suffix().toLower()=="pdf" )
        {

            qDebug() << fileName << fi.exists() << fi.suffix().toLower();
            slotOpenFileName(fileName);
            return;

        }


    }

}

//________________________________________________________________________
void MainWindow::slotAbout()
{
    QMessageBox::about(this, tr("About booksorg"),
                       tr("<p><b>booksorg %1</b></p><p>booksorg is a easy organizer for your PDF books "
                          "using qpdfview and poppler library.<p> See <a href=\"https://elkirtasse.sourceforge.net/\">elkirtasse.sourceforge.net</a> for more information.</p><p>&copy; About Zakaria</p>").arg(QApplication::applicationVersion()));

}

//________________________________________________________________________
void MainWindow::swichLanguage(QAction* act)
{

    QSettings settings;
    settings.beginGroup("Main");
    settings.setValue("Lang",act->data().toString());
    settings.endGroup();
    QMessageBox::information(this,"",tr("The language for this application has been changed.\n"
                                        "The change will take effect the next time the application is started."));



}

//________________________________________________________________________
void MainWindow::on_tabWidget_tabCloseRequested(int index)
{

    if( ui->tabWidget->indexOf(ui->tabBooks)==index){

        ui->tabWidget->removeTab(index);

    }else{

//        WidgetPdf   *w= qobject_cast<WidgetPdf *>( ui->tabWidget->widget(index));
//        if(w)
            delete ui->tabWidget->widget(index);

    }

    ui->tabWidget->setTabsClosable(ui->tabWidget->count()-1);

}

//________________________________________________________________________
void MainWindow::slotAddBookmark()
{

   // WidgetPdf   *w= qobject_cast<WidgetPdf *>( ui->tabWidget->currentWidget());
    if(widgetPdf())
       widgetPdf()->addBookmark();

}

//________________________________________________________________________
void MainWindow::bookmarkOpenDocument(const QString &fileName,int page,qreal left,qreal top)
{

    slotOpenFileName(fileName);
  //  WidgetPdf   *w= qobject_cast<WidgetPdf *>( ui->tabWidget->currentWidget());
    if(widgetPdf())
        widgetPdf()->jumpToPage(page,left,top);

}

//____________________________PDFWIDGET______________________________________
WidgetPdf *MainWindow::widgetPdf()
{
    WidgetPdf   *w= qobject_cast<WidgetPdf *>( ui->tabWidget->currentWidget());
    if(w)
        return w;
    return 0;
}

//void MainWindow::wPdfToggleContentVisible(bool arg)
//{
// //   WidgetPdf   *wPdf= qobject_cast<WidgetPdf *>( ui->tabWidget->currentWidget());
//    if(widgetPdf())
//        widgetPdf()->docToggleContentVisible(arg);
//}

void MainWindow::wPdfEmitFirstPage()
{

    if(widgetPdf())
        widgetPdf()->docEmitFirstPage();
}
void MainWindow::wPdfEmitPreviousPage()
{

    if(widgetPdf())
        widgetPdf()->docEmitPreviousPage();
}
void MainWindow::wPdfEmitNextPage()
{

    if(widgetPdf())
        widgetPdf()->docEmitNextPage();
}
void MainWindow::wPdfEmitLastPage()
{

    if(widgetPdf())
        widgetPdf()->docEmitLastPage();
}
void MainWindow::wPdfEmitAddBookmark()
{

    if(widgetPdf())
        widgetPdf()->docEmitAddBookmark();
}
void MainWindow::wPdfRefresh()
{

    if(widgetPdf())
        widgetPdf()->docRefresh();
}
void MainWindow::wPdfEmitZoomIn()
{

    if(widgetPdf())
        widgetPdf()->docEmitZoomIn();
}
void MainWindow::wPdfEmitZoomOut()
{

    if(widgetPdf())
        widgetPdf()->docEmitZoomOut();
}
void MainWindow::wPdfEmitSetRubberBand(bool arg)
{

    if(widgetPdf())
        widgetPdf()->docEmitSetRubberBand(arg);
}
void MainWindow::wPdfEmitSetInvertColors(bool arg)
{

    if(widgetPdf())
        widgetPdf()->docEmitSetInvertColors(arg);
}

void MainWindow::wPdfSetPage(int arg)
{

    if(widgetPdf())
        widgetPdf()->docSetPage(arg);
}

void MainWindow::wPdfSetContinousMode(bool arg)
{

        if(widgetPdf())
           widgetPdf()->docSetContinousMode(arg);



}

void MainWindow::wPdfSetFiteWith(int arg)
{

    if(widgetPdf())
      widgetPdf()->docSetFiteWith(arg);


}

void MainWindow::wPdfSetLayoutMode(int arg)
{

    if( widgetPdf())
        widgetPdf()->docSetLayoutMode(arg);

}


void MainWindow::showHideSearch()
{
    if(ui->tabWidget->currentWidget()==ui->tabBooks){
        if(mSearchTools->isHidden()){
            mSearchTools->setVisible(true);
            mSearchTools->show();
        }else
            mSearchTools->hide();

      }
    else if(widgetPdf())
        widgetPdf()->showHideSearch();
}


void MainWindow::showTabBooks()
{
    int i=  ui->tabWidget->indexOf(ui->tabBooks);
    if(i<0){

    ui->tabWidget->addTab(ui->tabBooks,EIcon::icon(BOOKS_TAB),tr("Books"));
  ui->tabWidget->setCurrentWidget(ui->tabBooks);
    }else{
         ui->tabWidget->setCurrentIndex(i);
    }

}
