/* Copyright © Abouzakaria
 * (c)GPL2 (c)GPL3
 *
 * This file is part of booksorg.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License version 2 or at your option version 3 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H
//#include "mysystemmodel.h"
#include "mymodel.h"
#include "actions.h"
#include "searchtools.h"
#include "widgetpdf.h"
#include "ratingwidget.h"
#include "widgetbookmark.h"
#include <QMainWindow>
#include <QActionGroup>
#include <QToolBar>
namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

signals:
 void   curPageChanged(int);
private slots:

    void slotOpen();
    void slotOpenFileName(const QString &fileName);
    QString bookTitle(const QString &fileName, WidgetPdf *w=0);
    void slotOpenWith();
    void addBook();
     void savRating(int r, QString file);
    void setCurentTab(int);
    void scanFolder();
    void editBook();
    void deletSelectedIndexes();
    void changeAuthor();
    void changeGroup();
    void moveItems(bool authors);
    void goRootPath();
    void slotItemActivated(const QModelIndex & index);
    void slotcurrentRowChanged(const QModelIndex &current, const QModelIndex &);
    void customContextMenu(QPoint);
    void setupTools();
    void toggleView(QAction* action);
    void toggleRootPath(QAction* action);
    void pathChanged(QString);
    QModelIndex currentIndex();
    void goSearch(const QString &arg);
    void updateRecentFileActions();
    void openRecentFile();
    void slotAbout();
    void swichLanguage(QAction* act);
    void slotAddBookmark();
    void bookmarkOpenDocument(const QString &fileName,int page,qreal left,qreal top);
    void on_tabWidget_tabCloseRequested(int index);
WidgetPdf *widgetPdf();
void showTabBooks();
//void wPdfToggleContentVisible(bool arg);
void wPdfEmitFirstPage();
void wPdfEmitPreviousPage();
void wPdfEmitNextPage();
void wPdfEmitLastPage();
void wPdfEmitAddBookmark();
void wPdfRefresh();
void wPdfEmitZoomIn();
void wPdfEmitZoomOut();
void wPdfEmitSetRubberBand(bool arg);
void wPdfEmitSetInvertColors(bool arg);
void wPdfSetPage(int arg);
void wPdfSetContinousMode(bool arg);
void wPdfSetFiteWith(int arg);
void wPdfSetLayoutMode(int arg);
void showHideSearch();
private:

    void dragEnterEvent(QDragEnterEvent *e);
    void dropEvent(QDropEvent *e);

    Ui::MainWindow *ui;

    enum ViewMode{
        IconView,
        DetailView
    };

 //   QToolBar *mToolBarBooks;
  RatingWidget *mRatingWidget;
  QAction *mActBooksTab;
    QAction *mActScanFolder;
    QAction *mActDelete;
    QAction *mActMove;
    QAction *mActOpen;
    QAction *mActOpenWith;

    QAction *mActMoveToGroup;
    QAction *mActMoveToAuthor;
    QAction *mActAddBook;
    QAction *mActEditBook;
    QAction *actIconView;
    QAction *actDetailView;
    QAction *mActGroopPath;
    QAction *mActAutorPath;
    QAction *mActFavoPath;
    QAction *mActRootPath;
    QAction *mActAbout;
     QAction *mActFind;
    QActionGroup  *alignViewMode;
    QActionGroup  *alignRootPath;

    QList<QAction*>mListEditBooks;
    QList<QAction*>mListViewBooks;

    QList<QAction*>mListEditPdf;
    QList<QAction*>mListViewPdf;

    enum { MaxRecentFiles = 7 };
    QAction *recentFileActs[MaxRecentFiles];
QMenu *menuRecent;
    QAction *mActRecents;
    MyModel *mModel;

    SearchTools *mSearchTools;
    WidgetPdf *mWidgetPdf;
    WidgetBookmark *mWidgetBookmark;

    Actions *mActions;
};

#endif // MAINWINDOW_H
