﻿/* Copyright © Abouzakaria
 * (c)GPL2 (c)GPL3
 *
 * This file is part of booksorg.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License version 2 or at your option version 3 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */
#include "mymodel.h"
#include "treeitem.h"
#include <QDebug>
#include <QSqlError>
#include <QSqlQuery>
#include <QPainter>
#include <QMessageBox>
#include <QFontInfo>

//_____________________________________________________
MyModel::MyModel(QObject *parent) :
    QStandardItemModel(parent)
{
    insertColumns(0, 5);
    db = QSqlDatabase::addDatabase("QSQLITE");
    db.setDatabaseName(BooksPath+"/books.db");


    QString txt="no database: ";
    if(!db.open())
    {
        txt+=db.lastError().text();
        qDebug()<<txt;
    }

    //! [Set up table  book]

    QSqlQuery query(db);

    //AUTOINCREMENT
    query.exec("CREATE TABLE 'main'.'books' ("
               "'id' INTEGER PRIMARY KEY  NOT NULL,"
               "'title' TEXT,"
               "'author' TEXT,"
               "'groups' TEXT,"
               "'favo' INTEGER NOT NULL DEFAULT (0),"
               "'icon' TEXT,"
               "'comment' TEXT,"
               "'properties' TEXT,"
               "'exec' TEXT,"
               "'page' INTEGER NOT NULL DEFAULT (1)"
               ");");


    //tmp
    /*
    addBook(trUtf8("جزء اصل الاسلام والرسالة "), trUtf8("yahiaz"),
            trUtf8(""), trUtf8("جزء_أصل_دين_الإسلام.png"),
            trUtf8("ghhhhhhh"), trUtf8("dff ghhh vvv"),
            trUtf8("/home/yahia/التنزيلات/جزء_أصل_دين_الإسلام.pdf"));

    addBook(trUtf8("شبهة أن بعض أهل العلم لم يكفروا ابن عربي  "), trUtf8("yahiaz"),
            trUtf8(""), trUtf8("شبهة_أن_بعض_أهل_العلم_لم_يكفروا_ابن_عربي1.png"),
            trUtf8("ghhhhhhh"), trUtf8("dff ghhh vvv"),
            trUtf8("/home/yahia/التنزيلات/جزء_أصل_دين_الإسلام.pdf"));

    addBook(trUtf8("جزء اصل الاسلام والرسالة "), trUtf8("mohamed"),
            trUtf8("halm"), trUtf8("جزء_أصل_دين_الإسلام.png"),
            trUtf8("ghhhhhhh"), trUtf8("dff ghhh vvv"),
            trUtf8("/home/yahia/التنزيلات/جزء_أصل_دين_الإسلام.pdf"));

    addBook(trUtf8("شبهة أن بعض أهل العلم لم يكفروا ابن عربي  "), trUtf8("mohamed"),
            trUtf8("halm"), trUtf8("شبهة_أن_بعض_أهل_العلم_لم_يكفروا_ابن_عربي1.png"),
            trUtf8("ghhhhhhh"), trUtf8("dff ghhh vvv"),
            trUtf8("/home/yahia/التنزيلات/جزء_أصل_دين_الإسلام.pdf"));

*/
    init();

    //  setCategory(CAT_GROUP);

}

//___________________________________________________________________
QVariant MyModel::data(const QModelIndex &index, int role) const
{

    if (!index.isValid()) return QVariant();
    /*
    if(!isRoot(index)){
        QStandardItem* curItem=item(index.row(),0);

        if(index.column() == 1){
             if (role == Qt::DisplayRole)return curItem->data(Qt::UserRole+AUTHOR);
        }else if(index.column() == 2){
            if (role == Qt::DisplayRole)return  curItem->data(Qt::UserRole +COMMENT);//comment
        }else if(index.column() == 3){

            if (role == Qt::DisplayRole)return  curItem->data(Qt::UserRole +FAVO).toString();//favo
            if (role == Qt::DecorationRole)
                return QIcon(QString(":/icons/%1.png")
                             .arg(curItem->data(Qt::UserRole +FAVO).toString()));

        }
    }
    */
    return QStandardItemModel::data(index,role);

}

//____________________________________________________________________________________
QVariant MyModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(orientation==Qt::Horizontal)
        if(role == Qt::DisplayRole)
            switch(section)
            {
            case 0: return tr("Name");
            case 1: return tr("Author");
            case 2: return tr("Comment");
            case 3: return tr("Group");
            case 4: return tr("Rating");

            default: return QVariant();
            }

    return QVariant();

}

//_____________________________________________________________________
void  MyModel::addBook(const QString &title,   const QString &author,
                       const QString &group,   const QString &icon,
                       const QString &comment, const QString &properties,
                       const QString &exec)
{

    if(!this->title(exec).isEmpty()){
        qDebug()<<"This file exists :"<<exec;
        QFile::remove(thumbnails+"/"+icon);
        return;
    }

    QSqlQuery query;

/****************************************************************************
 *                                                                                                                                                    *
 * insert columns into Books in order                                                                                                *
 * id=0 1=title 2=author  3=groups 4=favo  5=icon 6=comment 7=properties 8=execs  9=page     *
 *                                                                                                                                                    *
 * **************************************************************************/

    QString txt=QString("insert into books values(NULL,'%1','%2','%3','%4','%5','%6','%7','%8','%9')")
            .arg(title)
            .arg(author!=tr("Unknown") ? author:QString())
            .arg(group!=tr("Unknown") ? group:QString())
            .arg("0")
            .arg(icon)
            .arg(comment)
            .arg(properties)
            .arg(exec)
            .arg("1")
            ;

    query.exec(txt);

        if(!mListAuthors.contains(author)&& !author.isEmpty())
        mListAuthors.append(author);

    if(!mListGroups.contains(group) && !group.isEmpty())
        mListGroups.append(group);


}

//_____________________________________________________
void  MyModel::updateBook(const QModelIndex &index,
                          const QString &title,
                          const QString &author,
                          const QString &group,
                          const QString &comment,
                          const QString &path,
                          const int &favo)
{




    QString txt=QString("UPDATE books"
                        " SET title = '%1',"
                        "author = '%2',"
                        "groups = '%3',"
                        "comment = '%4',"
                        "exec = '%5',"
                        "favo = '%6'"
                        " WHERE id = '%7'").arg(title)
            .arg(author!=tr("Unknown") ? author: QString()).arg(group!=tr("Unknown") ? group : QString()).arg(comment)
            .arg(path).arg(favo).arg(id(index));

    if(!mListGroups.contains(group) && !group.isEmpty())
        mListGroups.append(group);

    if(!mListAuthors.contains(author) && !author.isEmpty())
        mListAuthors.append(author);

    QSqlQuery query(db);
    if(query.exec(txt))
        refreshPath();

}

//__________________________________________________________________________
void  MyModel::updateCategiry(const QString &oldTitle,const QString &title)
{
    QString column;
    switch (mCategoryName)
    {

    case CAT_GROUP: column="groups"; break;
    case CAT_FAVORIT:               return;
    case CAT_AUTHOR: column="author";break;
    default: break;

    }

    QString old_title=oldTitle;
    if(old_title==tr("Unknown"))
        old_title=QString();

    QString txt=QString("UPDATE books"
                        " SET %1 = '%2'"
                        " WHERE %1 = '%3'").arg(column)
            .arg(title).arg(old_title);

    QSqlQuery query(db);
    int i=-1;
    if(query.exec(txt)){
        switch (mCategoryName)
        {
        case CAT_GROUP:
            i=mListGroups.indexOf(old_title);
            if(i!=-1)
                mListGroups.replace(i,title);
            else
                mListGroups.append(title);
            break;

        case CAT_AUTHOR:
            i=mListAuthors.indexOf(old_title);
            if(i!=-1)
                mListAuthors.replace(i,title);
            else
                mListAuthors.append(title);
            break;
        }
        refreshPath();
    }
}

void MyModel::init()
{

    mListAuthors.clear();
    mListFavo.clear();
    mListGroups.clear();

    mListAuthors.append(tr("Unknown"));
    mListGroups.append(tr("Unknown"));

    for (int i = 0; i < 6; ++i)
        mListFavo.append(QString::number(i));

    QSqlQuery query;
    query.exec("SELECT DISTINCT author FROM books");

    while(query.next())
    {

        QString author=query.value(0).toString();
        if(!mListAuthors.contains(author)&& !author.isEmpty())
            mListAuthors.append(author);

    }

    query.exec("SELECT DISTINCT groups FROM books");

    while(query.next())
    {

        QString group=query.value(0).toString();

        if(!mListGroups.contains(group)&& !group.isEmpty())
            mListGroups.append(group);

    }



}

//_____________________________________________________
void MyModel::setCategory(int cat)
{

    mIsCategory=true;
    mRootPath=QString();
    //  qDebug()<<invisibleRootItem()->rowCount();
    //    int count=invisibleRootItem()->rowCount();
    //    for (int i = 0; i < count; ++i) {
    //          invisibleRootItem()->removeRow(0);
    //    }
    clear();
    invisibleRootItem()->insertColumns(0, 5);
    QString icon;
    mCategoryName=cat;
    QStringList list;

    switch (cat) {
    case CAT_GROUP:
        list=mListGroups;
        icon="group128";
        break;

    case CAT_FAVORIT:
        list=mListFavo;
        icon="favo_";
        break;

    case CAT_AUTHOR:
        list=mListAuthors;
        icon="author128";
        break;

    default:
        break;
    }

    foreach (QString s, list)
    {
        if(s.isEmpty()||s.isNull())
            continue;
        QStandardItem* item = new QStandardItem();
        item->setText(s);

        if(mCategoryName==CAT_FAVORIT)
            item->setIcon(QIcon(QString(":/icons/%1.png").arg(icon+s)));
        else
            item->setIcon(QIcon(QString(":/icons/%1.png").arg(icon)));

        item->setData(true,Qt::UserRole+ISROOT);
        invisibleRootItem()-> appendRow(item);

    }
    emit pathChanged(pathToString());
}

//_____________________________________________________
void MyModel::setRootPath(const QString & name)
{
    mIsCategory=false;
    mRootPath=name;
    QString column;
    switch (mCategoryName)
    {
    case CAT_GROUP:
        column="groups";
        break;
    case CAT_FAVORIT:
        column="favo";
        break;
    case CAT_AUTHOR:
        column="author";
        break;
    default:
        break;
    }

    //    int count=invisibleRootItem()->rowCount();

    //    for (int i = 0; i < count; ++i)
    //    {
    //        invisibleRootItem()->removeRow(0);
    //    }

    //
    clear();



    QString txt=QString("select * from books WHERE %1='%2'")
            .arg(column).arg(name!=tr("Unknown") ? name:QString());
    qDebug()<<"Query exec :"<<txt;
    QSqlQuery query(txt);
    //  invisibleRootItem()->insertColumns(0, 4);
    while(query.next())
    {

        int id=query.value(ID).toInt();
        QString title=query.value(TITLE).toString();
        QString author=query.value(AUTHOR).toString();
        QString icon=query.value(ICON).toString();
        QString properties=query.value(PROPERTY).toString();
        QString exec=query.value(EXEC).toString();
        QString favo=query.value(FAVO).toString();
        QString group=query.value(GROUP).toString();
        QString comment=query.value(COMMENT).toString();

        QStandardItem* item = new QStandardItem;
        item->setText(title);
        item->setIcon(iconPix(icon));

        item->setData(false,Qt::UserRole+ISROOT);
        item->setData(exec,Qt::UserRole +EXEC);
        item->setData(icon,Qt::UserRole +ICON);
        item->setData(properties,Qt::UserRole +PROPERTY);
        item->setData(id,Qt::UserRole +ID);

        invisibleRootItem()->appendRow(QList<QStandardItem*>()<<item
                                       <<new QStandardItem(author=author!=QString()? author :tr("Unknown"))
                                       <<new QStandardItem(comment)
                                       <<new QStandardItem(group=group!=QString()? group :tr("Unknown"))
                                       <<new QStandardItem(QIcon(QString(":/icons/%1.png")
                                                                 .arg(favo)),favo));
        // invisibleRootItem()->appendRow(item);
    }

    emit pathChanged(pathToString());
}

//_____________________________________________________
void MyModel::setSearchPath(const QString & serachStr,bool machCase)
{
    mIsCategory=false;
    mRootPath=QString();
    // mCategory=CAT_SEARCH;
    clear();

    QString txt;
    if(machCase){
        txt=QString("select * from books WHERE title='%1'")
                .arg(serachStr);
    }else{
        txt=QString("select * from books WHERE title like '%%1%'")
                .arg(serachStr);
    }

    QSqlQuery query(txt);
   qDebug()<<"Query Search exec :"<<txt;
    while(query.next())
    {

        int id=query.value(ID).toInt();
        QString title=query.value(TITLE).toString();
        QString author=query.value(AUTHOR).toString();
        QString icon=query.value(ICON).toString();
        QString properties=query.value(PROPERTY).toString();
        QString exec=query.value(EXEC).toString();
        QString favo=query.value(FAVO).toString();
        QString group=query.value(GROUP).toString();
        QString comment=query.value(COMMENT).toString();

        QStandardItem* item = new QStandardItem;
        item->setText(title);
        item->setIcon(iconPix(icon));

        item->setData(false,Qt::UserRole+ISROOT);
        item->setData(exec,Qt::UserRole +EXEC);
        item->setData(icon,Qt::UserRole +ICON);
        item->setData(properties,Qt::UserRole +PROPERTY);
        item->setData(id,Qt::UserRole +ID);

        invisibleRootItem()->appendRow(QList<QStandardItem*>()<<item
                                       <<new QStandardItem(author)
                                       <<new QStandardItem(comment)
                                       <<new QStandardItem(group)
                                       <<new QStandardItem(QIcon(QString(":/icons/%1.png")
                                                                 .arg(favo)),favo));
        //         invisibleRootItem()->appendRow(item);
    }
    emit pathChanged(tr("Search/")+serachStr);



}

//_____________________________________________________
QIcon MyModel::iconPix(QString iconName){
    QPainter p;
    QPixmap pixIcon(260,260);
    QPixmap pix(thumbnails+"/"+iconName);
    pixIcon.fill(Qt::transparent);
    p.begin(&pixIcon);
    p.fillRect(0,0,259,259,QColor(229,239,255,120));

    QRect rect((260-pix.width())/2,(260-pix.height())/2,pix.width()-2,pix.height()-2);
    p.drawPixmap(rect,pix);

    p.setPen(Qt::lightGray);
    p.drawRect(0,0,260,260);
    p.setPen(Qt::gray);
    p.drawRect(-1,-1,259,259);
    p.setPen(Qt::gray);

    p.setOpacity(0.3);
    p.drawRect(-1,-1,260,260);

    p.end();
    return QIcon(pixIcon);
}
//_________________________________________________________________________
void MyModel::changeParent( const QModelIndex index,const QString &parentName,bool author)
{

    QString column;
    if(author){
        column="author";
    }else{
        column="groups";
    }


    QStandardItem* curItem=item(index.row(),0);

    if(curItem)
    {

        QString id= curItem->data(Qt::UserRole+ID).toString();
        QString txt=QString("UPDATE books"
                            " SET '%1' = '%2'"
                            " WHERE id = '%3'")
                .arg(column)
                .arg(parentName!=tr("Unknown") ? parentName:QString())
                .arg(id);

        QSqlQuery query;
        query.exec(txt);

        qDebug()<<txt;


        //update list
        if(author){
            if(!mListAuthors.contains(parentName))
                mListAuthors.append(parentName);
        }else{
            if(!mListGroups.contains(parentName))
                mListGroups.append(parentName);
        }

    }



}
//_________________________________________________________________________
//void MyModel::changeGroup( const QModelIndex index,const QString &group)
//{
//    QStandardItem* curItem=item(index.row(),0);

//    if(curItem)
//    {

//        QString id= curItem->data(Qt::UserRole+ID).toString();
//        QString txt=QString("UPDATE books"
//                            " SET groups = '%1'"
//                            " WHERE id = '%2'").arg(group)
//                .arg(id);
//        QSqlQuery query;
//        query.exec(txt);

//        qDebug()<<"Query changeGroup exec :"<<txt;
//    }
//}

//_________________________________________________________________________
void MyModel::deleteSelectedIndexes( const QModelIndexList selectedIndexes)
{
    QList<int>list;
    foreach (QModelIndex index, selectedIndexes)
    {
        if(!list.contains( index.row())){
            list.append(index.row());

            QStandardItem* curItem=item(index.row(),0);
            if(!curItem)continue;

                if(curItem->data(Qt::UserRole+ISROOT).toBool())
                {
                    QString column;
                    switch (mCategoryName)
                    {
                    case CAT_GROUP:  column="groups"; break;
                    case CAT_FAVORIT: column="favo";  break;
                    case CAT_AUTHOR:  column="author";break;
                    default: break;

                    }

                    // التأكد من حذف كل الكتب في المجموعة
                    QString title= curItem->text();
                    if(title==tr("Unknown")) title=QString();

                    if(confirmDeletion(title))
                    {
                        QSqlQuery query;

                        //----------------------------------------------------------1
                        QString text1=QString("select * from books WHERE %1='%2'")
                                .arg(column).arg(title);
                        query.exec(text1);
                        while(query.next())
                        {
                            QString iconName=query.value(ICON).toString();
                            QFile::remove(thumbnails+"/"+iconName);
                          //  qDebug()<<thumbnails+"/"+iconName;
                            qApp->processEvents();
                        }
                        //----------------------------------------------------------2
                        QString text2=QString("DELETE FROM books WHERE %1='%2'")
                                .arg(column).arg(title);

                        query.exec(text2);

                        switch (mCategoryName)
                        {
                        case CAT_GROUP:
                            mListGroups.removeOne(title);
                            break;

                        case CAT_AUTHOR:
                            mListAuthors.removeOne(title);
                            break;
                        default:
                            break;
                        }
                    }

                }
                else
                {

                    QString id= curItem->data(Qt::UserRole+ID).toString();
                    QString iconName=curItem->data(Qt::UserRole+ICON).toString();

                    //delete item data
                    QString txt=QString("DELETE FROM books WHERE id='%1'").arg(id);
                    QSqlQuery query;
                    query.exec(txt);
                    //remove item image
                    QFile::remove(thumbnails+"/"+iconName);
                }


        }

    }

    refreshPath();

}

//_____________________________________________________
bool MyModel::confirmDeletion(const QString &title)
{
    int ret = QMessageBox::warning(0, tr(""),
                                   tr("Do you want to delete All books in %1 ?.").arg(title),
                                   QMessageBox::Yes | QMessageBox::No,
                                   QMessageBox::Yes);

    if(ret==QMessageBox::Yes)
        return true;

    return false;

}

//_____________________________________________________
void MyModel::refreshPath()
{
    if(mIsCategory)
        setCategory(mCategoryName);
    else
        setRootPath(mRootPath);
}

//_____________________________________________________INFORMATION
bool MyModel::isRoot(const QModelIndex &index) const
{

    if(!index.isValid())return false;
    QStandardItem* curItem=item(index.row(),0);
    return curItem->data(Qt::UserRole+ISROOT).toBool();

}
//_____________________________________________________INFORMATION
QString MyModel::title(const QString &file)const
{

    QString   txt=QString("select title from books WHERE exec='%1'")
            .arg(file);
    QSqlQuery query;
    query.exec(txt);
    while(query.next())
    {
        return query.value(0).toString();
    }
    return QString();

}

//_____________________________________________________INFORMATION
QString MyModel::title(const QModelIndex &index)const
{

    if(!index.isValid())return QString();

    QStandardItem* curItem=item(index.row(),0);
    if(curItem)
        return curItem->text();
    return QString();

}


//_____________________________________________________INFORMATION
QString MyModel::author(const QModelIndex &index)const
{

    if(!index.isValid())return QString();

    QStandardItem* curItem=item(index.row(),1);
    if(curItem)
        return curItem->text();
    return QString();

}
//_____________________________________________________INFORMATION
QString MyModel::comment(const QModelIndex &index)const
{

    if(!index.isValid())return QString();

    QStandardItem* curItem=item(index.row(),2);
    if(curItem)
        return curItem->text();
    return QString();

}
//_____________________________________________________INFORMATION
QString MyModel::group(const QModelIndex &index)const
{

    if(index.isValid()){
        QStandardItem* curItem=item(index.row(),3);
        if(curItem){
            QString name=curItem->text();
            return name=name!=QString()? name :tr("Unknown");
        }
    }
    return QString();

}
//_____________________________________________________INFORMATION
QString MyModel::favorite(const QModelIndex &index)const
{

    if(!index.isValid())return QString();
    QStandardItem* curItem=item(index.row(),4);
    if(curItem)
        return curItem->text();

    return QString();

}


//_____________________________________________________INFORMATION
QString MyModel::properties(const QModelIndex &index)const
{

    if(!index.isValid())return QString();

    QStandardItem* curItem=item(index.row(),0);
    if(curItem)
        return curItem->data(Qt::UserRole+PROPERTY).toString();
    return QString();

}
//_____________________________________________________INFORMATION
QString MyModel::path(const QModelIndex &index)const
{
    if(!index.isValid())return QString();

    QStandardItem* curItem=item(index.row(),0);
    if(curItem)
        return curItem->data(Qt::UserRole+EXEC).toString();
    return QString();

}
//_____________________________________________________INFORMATION
QString MyModel::pixmap(const QModelIndex &index)const
{

    if(!index.isValid())return QString();
    QStandardItem* curItem=item(index.row(),0);
    if(curItem)
        return curItem->data(Qt::UserRole+ICON).toString();
    return QString();

}
//_____________________________________________________INFORMATION
QString MyModel::id(const QModelIndex &index)const
{

    if(!index.isValid())return QString();

    QStandardItem* curItem=item(index.row(),0);
    if(curItem)
        return curItem->data(Qt::UserRole+ID).toString();
    return QString();

}

//_____________________________________________________INFORMATION
int MyModel::page(const QString &file)const
{

    QString   txt=QString("select page from books WHERE exec='%1'")
            .arg(file);
    QSqlQuery query;
    query.exec(txt);
    while(query.next())
    {
        return query.value(0).toInt();
    }

    return 1;

}

void MyModel::savePage(const QString &file,int page)const
{
    QString txt=QString("UPDATE books"
                        " SET page = '%1'"
                        " WHERE exec = '%2'")
            .arg(page).arg(file);

   qDebug()<<"Query  exec :"<<txt;


    QSqlQuery query(db);
    if(query.exec(txt))
        qDebug()<<"page saved"<<page;
}
void MyModel::saveRating(int rat,const QString &file)
{
    QString txt=QString("UPDATE books"
                        " SET favo = '%1'"
                        " WHERE exec = '%2'")
            .arg(rat).arg(file);

   qDebug()<<"Query  exec :"<<file;


    QSqlQuery query(db);
    if(query.exec(txt)){
     refreshPath();
    }

}


//_____________________________________________________INFORMATION
QString MyModel::pathToString()
{
    QString path;
    switch (mCategoryName)
    {
    case CAT_GROUP:   path=tr("Groups"); break;
    case CAT_FAVORIT: path=tr("Ratings");break;
    case CAT_AUTHOR:  path=tr("Authors");break;
    case CAT_SEARCH:  path=tr("Search"); break;
    default: break;
    }

    return path+"/"+mRootPath;
}
