/* Copyright © Abouzakaria
 * (c)GPL2 (c)GPL3
 *
 * This file is part of booksorg.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License version 2 or at your option version 3 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef MYMODEL_H
#define MYMODEL_H
#include "treeitem.h"

#include <QStandardItemModel>
#include <QApplication>
#include <QDir>

#include <QSqlDatabase>
#define BooksPath QDir::homePath()+"/."+QApplication::applicationName()
#define thumbnails QDir::homePath()+"/."+QApplication::applicationName()+"/thumbnails"

enum CateGory{
    CAT_GROUP,
    CAT_AUTHOR,
    CAT_FAVORIT,
    CAT_SEARCH
};

class MyModel : public QStandardItemModel
{
    Q_OBJECT
public:

    explicit MyModel(QObject *parent = 0);
    QVariant data(const QModelIndex &index, int role) const;
    int category(){return mCategoryName;}

protected:

    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    int columnCount(const QModelIndex &) const{return 5;}

signals:
    void pathChanged(QString);

public slots:

    void addBook(const QString &title, const QString &author,
                 const QString &group, const QString &icon,
                 const QString &comment, const QString &properties,
                 const QString &exec);

    void updateBook(const QModelIndex &index, const QString &title,
                    const QString &author,
                    const QString &group,
                    const QString &comment,
                    const QString &path, const int &favo);
    void  updateCategiry(const QString &oldTitle, const QString &title);
    bool isRoot(const QModelIndex &index)const;
    bool isCategory(){return mIsCategory;}
    void refreshPath();
    void setCategory(int cat);
    void setRootPath(const QString & name);
    void setSearchPath(const QString & serachStr,bool machCase=false);
    QIcon iconPix(QString iconName);
    void deleteSelectedIndexes( const QModelIndexList selectedIndexes);
    void changeParent(const QModelIndex index, const QString &parentName, bool author);
//    void changeGroup(const QModelIndex index, const QString &group);

    QString title(const QModelIndex &index)const;
    QString title(const QString &file)const;
    QString comment(const QModelIndex &index)const;
    QString properties(const QModelIndex &index)const;
    QString author(const QModelIndex &index)const;
    QString favorite(const QModelIndex &index)const;
    QString pixmap(const QModelIndex &index)const;
    QString id(const QModelIndex &index)const;
    QString path(const QModelIndex &index) const;
    QString group(const QModelIndex &index)const;
    int page(const QString &file)const;
    void savePage(const QString &file, int page)const;
     void saveRating(int rat, const QString &file);
    QStringList authorList()const {return mListAuthors;}
    QStringList groupList()const {return mListGroups;}

    QString rootPath() const  {return mRootPath;}
    //    void remove(QList<QModelIndex>list);
    QString pathToString();
private:

    enum Columns{
        ID,
        TITLE,
        AUTHOR,
        GROUP,
        FAVO,
        ICON,
        COMMENT,
        PROPERTY,
        EXEC,
        ISROOT
    };
    QSqlDatabase  db;
    int mCategoryName;
    bool mIsCategory;
    QStringList mListFavo;
    QStringList mListAuthors;
    QStringList mListGroups;
    QString mRootPath;

private slots:
    void init();

    bool confirmDeletion(const QString &title);

};

#endif // MYMODEL_H
